package egotham.flexibleloadcontrol;

import egotham.sdi.model.DiscreteValue;
import egotham.sdi.model.RealValue;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Test Implementation of the InOutAll class. It is able to control all inputs
 * and output methods
 *
 * @author thaddauh
 */
public class InOutAllTest implements InOutAll {

    private List<RealValue> consumptionData;
    private Boolean[] relays;
    private List<DiscreteValue> fullConsumption;
    private long[][] tariff;
    private double[] temperature;
    private double ConsumptionCap;
    private long time;

    /**
     * Constructor of InOutAllTest. all variables are initialized, memory
     * arranged etc.
     *
     * @param Number_Of_Relays Number of relays for utilities
     * @param number_of_thermostats Number of Thermostats used
     */
    public InOutAllTest(int Number_Of_Relays, int number_of_thermostats) {
        this.consumptionData = new ArrayList<>();
        this.relays = new Boolean[Number_Of_Relays];
        Arrays.fill(relays, Boolean.FALSE);
        this.temperature = new double[number_of_thermostats];
        Arrays.fill(temperature, 0);
        ConsumptionCap = 0;
        this.fullConsumption = new ArrayList<>();
        this.time = 0;
    }

    /**
     * Inputs don't matter, returns data from setConsumptionData(...)
     *
     * @param relayNumber
     * @param startTime
     * @param endTime
     * @param resultLimit
     * @return
     */
    @Override
    public List<RealValue> consumptionData(int relayNumber, long startTime, long endTime, int resultLimit) {
        return this.consumptionData;
    }

    /**
     * creates new RealValue objects and places them into the ConsumptionData
     * List. The timestamps are being matched with the data one by one. If the
     * timestamps are non existent or run out the current timestamp is used.
     *
     * @param timestamps
     * @param values
	*
     */
    public void setConsumptionData(long[] timestamps, double[] values) {
        this.consumptionData.clear();
        for (int i = 0; i < values.length; i++) {
            if (i <= timestamps.length) {
                this.consumptionData.add(new RealValue(timestamps[i], values[i]));
            } else {
                consumptionData.add(new RealValue(values[i]));

            }
        }

    }

    @Override
    public boolean activateRelay(int relayNumber) throws Exception {
        if (relays[relayNumber - 1]) {
            return false;
        } else {
            relays[relayNumber - 1] = true;
            return true;

        }
    }

    @Override
    public boolean deactivateRelay(int relayNumber) throws Exception {
        if (this.relays[relayNumber - 1]) {
            this.relays[relayNumber - 1] = true;
            return true;

        } else {
            return false;
        }
    }

    @Override
    public boolean statusRelay(int relayNumber) throws Exception {
        return this.relays[relayNumber - 1];
    }

    @Override
    public List<DiscreteValue> fullConsumption(int N, long startTime, long endTime) throws Exception {
        return this.fullConsumption;
    }

    public void setfullConsumption(long[] timestamps, long[] values) {
        this.fullConsumption.clear();
        for (int i = 0; i < values.length; i++) {
            if (i <= timestamps.length) {
                fullConsumption.add(new DiscreteValue(timestamps[i], values[i]));
            } else {
                fullConsumption.add(new DiscreteValue(values[i]));
            }
        }

    }

    @Override
    public long[][] gettariff(long endTime) throws Exception {
        return this.tariff;
    }

    public void settariff(long[][] tariff) {
        this.tariff = tariff;

    }

    @Override
    public double getTemperature(int sensorNumber) {

        return this.temperature[sensorNumber];

    }

    public void setTemperature(int sensorNumber, double temperature) {
        this.temperature[sensorNumber] = temperature;

    }

    @Override
    public double getConsumptionCap() {

        return this.ConsumptionCap;

    }

    public void setConsumptionCap(double ConsumptionCap) {
        this.ConsumptionCap = ConsumptionCap;

    }

    public void settime(long time) {
        this.time = time;
    }

    @Override
    public long getCurrentTime() {
        return this.time;
    }

}
