package egotham.flexibleloadcontrol.Optimisation;

import egotham.flexibleloadcontrol.control.UtilityComparator;
import egotham.flexibleloadcontrol.control.UtilityControl;
import egotham.flexibleloadcontrol.SortingTechnique.SortingTechnique;

import java.util.*;

/**
 * Implementation of Optimisation class
 *
 * @author ted
 */
public class PricePredictionOptimisation implements Optimisation {

    private List<SortingTechnique> techniques;

    /**
     * Constructor inititalizing Lists
     */
    public PricePredictionOptimisation() {

        techniques = new ArrayList<>();
    }

    public List<SortingTechnique> getTechniques() {
        return techniques;
    }

    /**
     * Clears the List of techniques and sets the new ones in place. Order is
     * never changed and techniques executed in order
     *
     * @param techniques
     */
    public void setTechniques(List<SortingTechnique> techniques) {
        this.techniques.clear();
        this.techniques = techniques;
    }

    /**
     * Uses all utilities in utils and runs them through the sorting techniques.
     * The List of utilities is sorted for priorities before executing the
     * sorting techniques and sorting the active and inactive Lists after
     * completing them.
     *
     * @param utils List of all utilities
     * @param active Contains Utilities that should be activated after sorting
     * @param inactive Contains Utilities that should be deactivated after
     * sorting
     * @return
     */
    @Override
    public boolean optimise(List<UtilityControl> utils, List<UtilityControl> active, List<UtilityControl> inactive) {

        Collections.sort(utils, new UtilityComparator());
        active.clear();
        active.addAll(utils);
        List<UtilityControl> tmp = new ArrayList<>();
        for (SortingTechnique technique : this.techniques) {
            tmp.clear();
            tmp.addAll(technique.sort(active, utils));
            active.clear();
            active.addAll(tmp);

        }

        for (UtilityControl util : utils) {
            if (!active.contains(util)) {
                inactive.add(util);
            }
        }

        Collections.sort(active, new UtilityComparator());
        Collections.sort(inactive, new UtilityComparator());

        return true;

    }

}
