package egotham.flexibleloadcontrol;

import egotham.flexibleloadcontrol.control.UtilityControl;
import egotham.flexibleloadcontrol.Optimisation.Optimisation;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Root class for the Controller, controls all utilities and optimisations
 *
 * @author ted
 */
public class Scheduler {

    private List<UtilityControl> utilities;
    private List<UtilityControl> optimisedUtilities;
    private List<UtilityControl> inactiveUtilities;

    private Optimisation optimisation;

    /**
     * reads the optimised list of utilities and activates them and deactivates
     * the ones in the inactive list
     *
     * @return true if output has been applied correctly, correct switches
     * guaranteed inside the controller Both return values mean the action has
     * been performed
     */
    public boolean givePermissions() {
        Boolean returnvalue = true;
        if (this.utilities.size() != this.optimisedUtilities.size() + this.inactiveUtilities.size()) {
            System.out.println("Scheduler: Lists dont add up!");
            returnvalue = false;
        } else {
            if (!this.optimisedUtilities.isEmpty()) {
                for (UtilityControl optimisedUtility : this.optimisedUtilities) {
                    if (!optimisedUtility.permitUsage(true)) {
                        returnvalue = false;
                    }
                }
                for (UtilityControl inactiveUtility : this.inactiveUtilities) {
                    if (!inactiveUtility.permitUsage(false)) {
                        returnvalue = false;
                    }
                }
            }
        }

        return returnvalue;

    }

    /**
     * optimises the List of utilities to match the parameters
     *
     * @return returns true if the operation was successful
     */
    public boolean optimiseLists() {
        this.optimisation.optimise(this.utilities, this.optimisedUtilities, this.inactiveUtilities);

        return true;
    }

    /**
     *
     * @return all utilities as a list
     */
    public List<UtilityControl> getUtilities() {
        return utilities;
    }

    /**
     * removes all items from the utilities list and replaces them with the new
     * list
     *
     * @param utilities
     */
    public void setUtilities(List<UtilityControl> utilities) {
        this.utilities.clear();
        this.utilities.addAll(utilities);
    }

    /**
     * removes all items from the utilities list and replaces them with the new
     * items
     *
     * @param util
     */
    public void setUtilities(UtilityControl[] util) {
        this.utilities.clear();
        List<UtilityControl> utils = new ArrayList<>(Arrays.asList(util));

        this.utilities.addAll(utils); //Adds all utilities to the List of utilities

    }

    /**
     *
     * @return returns the optimisation object
     */
    public Optimisation getoptimisation() {
        return optimisation;
    }

    /**
     * Sets the optimisation type Optimisation object
     *
     * @param optimisation
     */
    public void setoptimisation(Optimisation optimisation) {

        this.optimisation = optimisation;
    }

    /**
     * Constructor initialising all Lists
     */
    public Scheduler() {
        this.utilities = new ArrayList<>();
        this.inactiveUtilities = new ArrayList<>();
        this.optimisedUtilities = new ArrayList<>();
    }

    ;
  /**
   * Constructor initialising all Lists and setting optimisation
   * @param optimisation  the optimisation object
   */    
    public Scheduler(Optimisation optimisation) {
        this.utilities = new ArrayList<>();
        this.inactiveUtilities = new ArrayList<>();
        this.optimisedUtilities = new ArrayList<>();
        this.optimisation = optimisation;

    }

}
