package egotham.flexibleloadcontrol;

/**
 * Settings class for the Controller,
 *
 * @author ted
 */
public class Settings {

    private int priceDifference;// Amount a customer is willing to pay more  in NOK
    private int lowTemperature;//lower bound of Temperature a customer is willing to have
    private int highTemperature; // upper bound of Temperature a customer is willing to have
    private int bestTemperature; // optimal temperature
    private int updateInterval;// update interval in minutes
    private int staticFuturePricerange;// range in the 24h price prediction that is sure to be static
    private long startUpTime;//timestamp when the system was started

    /**
     * Constructor for Settings.
     *
     * @param priceDifference Amount of money a customer is willing to pay more
     * for a higher comfort level
     * @param lowTemperature Lowest temperature in a room
     * @param highTemperature highest temperature in a room
     * @param bestTemperature optimal temperature a controller is aiming towards
     * @param updateInterval How many minutes between each update interval.
     * @param staticFuturePricerange How many of the prices in the forecast are
     * not going to change
     * @param startUpTime E-Gotham Timestamp with the time, when the first data
     * is to be expected
     */
    public Settings(int priceDifference, int lowTemperature, int highTemperature, int bestTemperature, int updateInterval, int staticFuturePricerange, long startUpTime) {
        this.priceDifference = priceDifference;
        this.lowTemperature = lowTemperature;
        this.highTemperature = highTemperature;
        this.bestTemperature = bestTemperature;
        this.updateInterval = updateInterval;
        this.staticFuturePricerange = staticFuturePricerange;
        this.startUpTime = startUpTime;
    }

    public long getStartUpTime() {
        return this.startUpTime;
    }

    public void setStartUpTime(long startUpTime) {
        this.startUpTime = startUpTime;
    }

    public int getUpdateInterval() {
        return this.updateInterval;
    }

    public int getStaticFuturePricerange() {
        return staticFuturePricerange;
    }

    public void setStaticFuturePricerange(int staticFuturePricerange) {
        this.staticFuturePricerange = staticFuturePricerange;
    }

    public void setUpdateInterval(int updateInterval) {
        this.updateInterval = updateInterval;
    }

    public int getPriceDifference() {
        return priceDifference;
    }

    public void setPriceDifference(int priceDifference) {
        this.priceDifference = priceDifference;
    }

    public int getLowTemperature() {
        return lowTemperature;
    }

    public void setLowTemperature(int lowTemperature) {
        this.lowTemperature = lowTemperature;
    }

    public int getHighTmeperature() {
        return highTemperature;
    }

    public void setHighTmeperature(int highTmeperature) {
        this.highTemperature = highTmeperature;
    }

    public int getBestTemperature() {
        return bestTemperature;
    }

    public void setBestTemperature(int bestTemperature) {
        this.bestTemperature = bestTemperature;
    }

}
