package egotham.flexibleloadcontrol.SortingTechnique;

import egotham.flexibleloadcontrol.control.UtilityControl;
import egotham.flexibleloadcontrol.InOutAll;
import egotham.flexibleloadcontrol.Settings;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Implementation of Sorting technique, deals with current and future prices
 *
 * @author ted
 */
public abstract class PricePredictSort implements SortingTechnique {

    private final InOutAll inoutall;
    private final Settings settings;

    public Settings getSettings() {
        return settings;
    }

    public InOutAll getInOutAll() {
        return inoutall;

    }

    private enum priceLevel {

        low, high, average
    }

    private String priceLeveltoString(priceLevel pl) {
        String returnValue = "";
        switch (pl) {
            case low:
                returnValue = "low";
                break;
            case high:
                returnValue = "high";
                break;
            case average:
                returnValue = "average";
                break;
        }
        return returnValue;
    }

    public PricePredictSort(InOutAll inoutall, Settings settings) {
        this.inoutall = inoutall;
        this.settings = settings;

    }

    /**
     * Checks corelation between priorities of utilities with current and future
     * prices.
     *
     * @param utilities List of Utilities to be optimised
     * @param totalUtilities List of all utilities in the system ( for power
     * evaluation)
     * @return List of Utilities that should be permitted to run
     */
    @Override
    public List<UtilityControl> sort(List<UtilityControl> utilities, List<UtilityControl> totalUtilities) {

        List<UtilityControl> returnValue = new ArrayList<>();
        returnValue.addAll(utilities);
        try {
            priceLevel now = getPricelevel(inoutall.getCurrentTime(), 0);
            priceLevel future = getPricelevel(inoutall.getCurrentTime(), 1);
            System.out.println("Price now: " + priceLeveltoString(now) + " Price future: " + priceLeveltoString(future));
            int highestPrio = 1;

            switch (now) {

                case low: {
                    if (future == priceLevel.high) {
                        highestPrio = 9;
                    } else {
                        highestPrio = 5;
                    }
                    break;
                }
                case average: {
                    if (future == priceLevel.high) {
                        highestPrio = 7;

                    } else {
                        highestPrio = 5;
                    }
                    break;
                }
                case high: {
                    if (future == priceLevel.high) {
                        highestPrio = 3;
                    } else {
                        highestPrio = 1;
                    }

                }

            }

            for (UtilityControl util : utilities) {
                if (util.getUrgency() > highestPrio) {
                    returnValue.remove(util);
                }

            }

        } catch (Exception ex) {
            Logger.getLogger(PricePredictSort.class.getName()).log(Level.SEVERE, null, ex);
        }
        return returnValue;
    }

    /**
     * Returns the price level of the hour specified in timestamp and which hour
     * (starting at 0), needed for future prices from the 24*2 prices the low
     * price is taken
     *
     * @param timestamp egotham timestamp
     * @param hour which hourly price is taken
     * @return price level (high, average, low) of the price specified
     * @throws Exception
     */
    private priceLevel getPricelevel(long timestamp, int hour) throws Exception {
        long[][] tariff;
        double[] values;
        tariff = inoutall.gettariff(timestamp);// price at timestamp

        values = calcMean();

        if (tariff[hour][1] < (values[0] - values[1])) {
            return priceLevel.low;
        } else if (tariff[hour][1] > (values[0] + values[1])) {
            return priceLevel.high;
        } else {
            return priceLevel.average;
        }

    }

    /**
     * Calculates the Mean value and standard deviation of the 100 previous
     * prices. If the data is not available the deviation is done over the
     * available data.
     *
     * @return the double[]
     */
    public abstract double[] calcMean();
}
