package egotham.flexibleloadcontrol;

import egotham.flexibleloadcontrol.Optimisation.PricePredictionOptimisation;
import egotham.flexibleloadcontrol.SortingTechnique.CapSort;
import egotham.flexibleloadcontrol.SortingTechnique.PricePredictSortTest;
import egotham.flexibleloadcontrol.SortingTechnique.SortingTechnique;
import egotham.flexibleloadcontrol.control.TestControl;
import egotham.flexibleloadcontrol.control.UtilityControl;
import egotham.sdi.Clock;
import java.util.Arrays;

/**
 * Test class for sorting algorithms.
 *
 * @author ted
 */
public class Testing {

    /**
     * Creates a testing environment with TestControl and InOutAllTest objects.
     * All inputs can be controlled.
     *
     * @param setpoint specifies which situation is executed
     *
     */
    public void TestSorting(int setpoint) {
        InOutAllTest inout = new InOutAllTest(6, 5); // 5 Heater + 1 Boiler, 5 Thermostats
        Settings sett = new Settings(0, 17, 23, 21, 10, 5, inout.getCurrentTime());
        TestControl heater1 = new TestControl(1);       //Initialize Utilities and create array
        TestControl heater2 = new TestControl(2);
        TestControl heater3 = new TestControl(3);
        TestControl heater4 = new TestControl(4);
        TestControl heater5 = new TestControl(5);
        TestControl boiler1 = new TestControl(6);
        UtilityControl[] arrU = {heater1, heater2, heater3, heater4, heater5, boiler1};

        SortingTechnique cap = new CapSort(inout, sett);    //Initialize sorting techniques
        PricePredictSortTest price = new PricePredictSortTest(inout, sett);
        SortingTechnique[] arrS = {cap, price};

        PricePredictionOptimisation opt = new PricePredictionOptimisation(); // Initialize Optimisation

        opt.setTechniques(Arrays.asList(arrS));         // Add techniques to getoptimisation

        Scheduler scheduler = new Scheduler();          // Initialize scheduler

        scheduler.setUtilities(arrU);                   // add Utilities to scheduler
        scheduler.setoptimisation(opt);                 // add Optimisation Object to scheduler
        long[] timestamps = new long[2];
        long[] values = new long[2];
        long[][] tariff = new long[2][3];
        /*-------------------------Initialisation End------------------------------------*/
        switch (setpoint) {
            case 1:
                System.out.println("");
                System.out.println("Case one, low/low");
                inout.setConsumptionCap(5000);
                inout.settime(Clock.getInstance().currentTimestamp());//time doesn't matter

                timestamps[0] = 1;
                timestamps[1] = 2;
                values[0] = 0;
                values[1] = 2000;

                inout.setfullConsumption(timestamps, values);   //set fullconsumption to 2000

                tariff[0][0] = 0;
                tariff[0][1] = 10;       // low price this hour
                tariff[0][2] = 10;       // high price this hour
                tariff[1][0] = 0;
                tariff[1][1] = 10;       // low price next hour
                tariff[1][2] = 10;       // high price next hour
                inout.settariff(tariff);                        // set tariff to 10/10

                price.setMeanStd(20, 5);

                heater1.setConsumption(200);//Utilities consumption
                heater2.setConsumption(300);
                heater3.setConsumption(150);
                heater4.setConsumption(250);
                heater5.setConsumption(100);
                boiler1.setConsumption(100);

                heater1.setUrgency(1);//Utilities urgencies set
                heater2.setUrgency(6);
                heater3.setUrgency(4);
                heater4.setUrgency(7);
                heater5.setUrgency(3);
                boiler1.setUrgency(5);

                break;
            case 2:
                System.out.println("");
                System.out.println("Case two, low/average");
                inout.setConsumptionCap(5000);
                inout.settime(Clock.getInstance().currentTimestamp());//time doesn't matter
                timestamps[0] = 1;
                timestamps[1] = 2;
                values[0] = 0;
                values[1] = 2000;

                inout.setfullConsumption(timestamps, values);   //set fullconsumption to 2000

                tariff[0][0] = 0;
                tariff[0][1] = 10;
                tariff[0][2] = 10;
                tariff[1][0] = 0;
                tariff[1][1] = 20;
                tariff[1][2] = 30;

                inout.settariff(tariff);                        // set tariff to 10/10

                price.setMeanStd(20, 5);

                heater1.setConsumption(200);//Utilities consumption
                heater2.setConsumption(300);
                heater3.setConsumption(150);
                heater4.setConsumption(250);
                heater5.setConsumption(100);
                boiler1.setConsumption(100);

                heater1.setUrgency(1);//Utilities urgencies set
                heater2.setUrgency(6);
                heater3.setUrgency(4);
                heater4.setUrgency(7);
                heater5.setUrgency(3);
                boiler1.setUrgency(5);
                break;

            case 3:
                System.out.println("");
                System.out.println("Case three, average/average");
                inout.setConsumptionCap(5000);
                inout.settime(Clock.getInstance().currentTimestamp());//time doesn't matter
                timestamps[0] = 1;
                timestamps[1] = 2;
                values[0] = 0;
                values[1] = 4000;

                inout.setfullConsumption(timestamps, values);   //set fullconsumption to 2000

                tariff[0][0] = 0;
                tariff[0][1] = 20;
                tariff[0][2] = 30;
                tariff[1][0] = 0;
                tariff[1][1] = 20;
                tariff[1][2] = 30;

                inout.settariff(tariff);                        // set tariff to 10/10

                price.setMeanStd(20, 5);

                heater1.setConsumption(200);//Utilities consumption
                heater2.setConsumption(300);
                heater3.setConsumption(150);
                heater4.setConsumption(250);
                heater5.setConsumption(100);
                boiler1.setConsumption(100);

                heater1.setUrgency(1);//Utilities urgencies set
                heater2.setUrgency(6);
                heater3.setUrgency(4);
                heater4.setUrgency(7);
                heater5.setUrgency(3);
                boiler1.setUrgency(5);
                break;
            case 4:
                System.out.println("");
                System.out.println("Case four, average/high");
                inout.setConsumptionCap(5000);
                inout.settime(Clock.getInstance().currentTimestamp());//time doesn't matter
                timestamps[0] = 1;
                timestamps[1] = 2;
                values[0] = 0;
                values[1] = 4000;

                inout.setfullConsumption(timestamps, values);   //set fullconsumption to 2000

                tariff[0][0] = 0;
                tariff[0][1] = 20;
                tariff[0][2] = 30;
                tariff[1][0] = 0;
                tariff[1][1] = 30;
                tariff[1][2] = 50;

                inout.settariff(tariff);                        // set tariff to 10/10

                price.setMeanStd(20, 5);

                heater1.setConsumption(200);//Utilities consumption
                heater2.setConsumption(300);
                heater3.setConsumption(150);
                heater4.setConsumption(250);
                heater5.setConsumption(100);
                boiler1.setConsumption(100);

                heater1.setUrgency(1);//Utilities urgencies set
                heater2.setUrgency(6);
                heater3.setUrgency(4);
                heater4.setUrgency(7);
                heater5.setUrgency(3);
                boiler1.setUrgency(5);
                break;
            case 5:
                System.out.println("");
                System.out.println("Case five, high/high");
                inout.setConsumptionCap(5000);
                inout.settime(Clock.getInstance().currentTimestamp());//time doesn't matter
                timestamps[0] = 1;
                timestamps[1] = 2;
                values[0] = 0;
                values[1] = 8000;

                inout.setfullConsumption(timestamps, values);   //set fullconsumption to 2000

                tariff[0][0] = 0;
                tariff[0][1] = 30;
                tariff[0][2] = 50;
                tariff[1][0] = 0;
                tariff[1][1] = 30;
                tariff[1][2] = 50;

                inout.settariff(tariff);                        // set tariff to 10/10

                price.setMeanStd(20, 5);

                heater1.setConsumption(200);//Utilities consumption
                heater2.setConsumption(300);
                heater3.setConsumption(150);
                heater4.setConsumption(250);
                heater5.setConsumption(100);
                boiler1.setConsumption(100);

                heater1.setUrgency(1);//Utilities urgencies set
                heater2.setUrgency(6);
                heater3.setUrgency(4);
                heater4.setUrgency(7);
                heater5.setUrgency(3);
                boiler1.setUrgency(5);
                break;

            case 6:
                System.out.println("");
                System.out.println("Case six, high/average");
                inout.setConsumptionCap(5000);
                inout.settime(Clock.getInstance().currentTimestamp());//time doesn't matter
                timestamps[0] = 1;
                timestamps[1] = 2;
                values[0] = 0;
                values[1] = 4000;

                inout.setfullConsumption(timestamps, values);   //set fullconsumption to 2000

                tariff[0][0] = 0;
                tariff[0][1] = 30;
                tariff[0][2] = 50;
                tariff[1][0] = 0;
                tariff[1][1] = 20;
                tariff[1][2] = 30;

                inout.settariff(tariff);                        // set tariff to 10/10

                price.setMeanStd(20, 5);

                heater1.setConsumption(200);//Utilities consumption
                heater2.setConsumption(300);
                heater3.setConsumption(150);
                heater4.setConsumption(250);
                heater5.setConsumption(100);
                boiler1.setConsumption(100);

                heater1.setUrgency(1);//Utilities urgencies set
                heater2.setUrgency(6);
                heater3.setUrgency(4);
                heater4.setUrgency(7);
                heater5.setUrgency(3);
                boiler1.setUrgency(5);
                break;
            case 7:
                System.out.println("");
                System.out.println("Case seven, average/low");
                inout.setConsumptionCap(5000);
                inout.settime(Clock.getInstance().currentTimestamp());//time doesn't matter
                timestamps[0] = 1;
                timestamps[1] = 2;
                values[0] = 0;
                values[1] = 4000;

                inout.setfullConsumption(timestamps, values);   //set fullconsumption to 2000

                tariff[0][0] = 0;
                tariff[0][1] = 20;
                tariff[0][2] = 30;
                tariff[1][0] = 0;
                tariff[1][1] = 10;
                tariff[1][2] = 10;

                inout.settariff(tariff);                        // set tariff to 10/10

                price.setMeanStd(20, 5);

                heater1.setConsumption(200);//Utilities consumption
                heater2.setConsumption(300);
                heater3.setConsumption(150);
                heater4.setConsumption(250);
                heater5.setConsumption(100);
                boiler1.setConsumption(100);

                heater1.setUrgency(1);//Utilities urgencies set
                heater2.setUrgency(6);
                heater3.setUrgency(4);
                heater4.setUrgency(7);
                heater5.setUrgency(3);
                boiler1.setUrgency(5);
                break;
            case 8:
                System.out.println("");
                System.out.println("Case eight, low/high");
                inout.setConsumptionCap(5000);
                inout.settime(Clock.getInstance().currentTimestamp());//time doesn't matter
                timestamps[0] = 1;
                timestamps[1] = 2;
                values[0] = 0;
                values[1] = 2000;

                inout.setfullConsumption(timestamps, values);   //set fullconsumption to 2000

                tariff[0][0] = 0;
                tariff[0][1] = 10;
                tariff[0][2] = 10;
                tariff[1][0] = 0;
                tariff[1][1] = 30;
                tariff[1][2] = 50;

                inout.settariff(tariff);                        // set tariff to 10/10

                price.setMeanStd(20, 5);

                heater1.setConsumption(200);//Utilities consumption
                heater2.setConsumption(300);
                heater3.setConsumption(150);
                heater4.setConsumption(250);
                heater5.setConsumption(100);
                boiler1.setConsumption(100);

                heater1.setUrgency(1);//Utilities urgencies set
                heater2.setUrgency(6);
                heater3.setUrgency(4);
                heater4.setUrgency(7);
                heater5.setUrgency(3);
                boiler1.setUrgency(5);
                break;
            case 9:
                System.out.println("");
                System.out.println("");
                System.out.println("Case nine, high/low");
                inout.setConsumptionCap(5000);
                inout.settime(Clock.getInstance().currentTimestamp());//time doesn't matter
                timestamps[0] = 1;
                timestamps[1] = 2;
                values[0] = 0;
                values[1] = 8000;

                inout.setfullConsumption(timestamps, values);   //set fullconsumption to 2000

                tariff[0][0] = 0;
                tariff[0][1] = 30;
                tariff[0][2] = 50;
                tariff[1][0] = 0;
                tariff[1][1] = 10;
                tariff[1][2] = 10;

                inout.settariff(tariff);                        // set tariff to 10/10

                price.setMeanStd(20, 5);

                heater1.setConsumption(200);//Utilities consumption
                heater2.setConsumption(300);
                heater3.setConsumption(150);
                heater4.setConsumption(250);
                heater5.setConsumption(100);
                boiler1.setConsumption(100);

                heater1.setUrgency(1);//Utilities urgencies set
                heater2.setUrgency(6);
                heater3.setUrgency(4);
                heater4.setUrgency(7);
                heater5.setUrgency(3);
                boiler1.setUrgency(5);
                break;
        }

        scheduler.optimiseLists();

        scheduler.givePermissions();
        System.out.println("Test done on setpoint" + String.valueOf(setpoint));

    }
}
