package egotham.flexibleloadcontrol.control;

/**
 * Interface for utilities like heaters and boilers.
 *
 * @author ted
 *
 */
public interface UtilityControl {

    /**
     * When the function is called the utility starts working or stops,
     * depending on input
     *
     * @param use either gives control (true) or turns it off (false)
     * @return gives true or false depending on success
     */
    public boolean permitUsage(boolean use);

    /**
     * Creates a schedule and finds the urgency level
     *
     *
     * @return urgency returns a value of how urgent the utility estimates its
     * need to be scheduled
     */
    public int getUrgency();

    /**
     * Runs an estimation on previous data to find the right value.
     *
     * @return returns an estimate of consumption for the next hour in w*h
     */
    public double getConsumptionEstimation();

}
